/*
    This file is part of ben -- a task scheduler for distributed computations
    Copyright (C) 2012-2024  Laurent Poirrier

    ben is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "xmem.h"
#include "array.h"

// ***************************************************************************
// 
// ***************************************************************************
void array_init(array *a, size_t cell, size_t gran)
{
	size_t m;

	if (gran == 0) {
		a->mask = 0;
	} else {
		m = 2;
		
		while ((m < gran) && (m))
			m <<= 1;

		if (m)
			a->mask = m - 1;
		else
			a->mask = 0;
	}
	
	a->n = 0;
	a->alloc = 0;
	a->cell = cell;
	a->data = NULL;
}

void array_clear(array *a)
{
	xfree(a->data);
}

void array_reset(array *a)
{
	a->n = 0;
}

// ***************************************************************************
// 
// ***************************************************************************
void array_alloc(array *a, size_t n)
{
	if (n <= a->alloc)
		return;
	
	if (a->mask) {
		a->alloc = (n + a->mask) & (~a->mask);
	} else {
		a->alloc = 1;
		
		while ((a->alloc < n) && (a->alloc))
			a->alloc <<= 1;
		
		if (!a->alloc) {
			fprintf(stderr, "array_alloc(): %zd too large\n", n);
			exit(1);
		}
	}
	
	a->data = xrealloc(a->data, a->alloc * a->cell);
}

void *array_get(array *a, size_t n)
{
	array_alloc(a, n);
	a->n = n;
	return(a->data);
}


// ***************************************************************************
// 
// ***************************************************************************
void array_append(array *a, void *data, size_t n)
{
	size_t o;
	
	o = a->n;
	array_get(a, o + n);
	
	memcpy(a->data + (o * a->cell), data, n * a->cell);
}


