/*
    This file is part of ben -- a task scheduler for distributed computations
    Copyright (C) 2012-2024  Laurent Poirrier

    ben is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BEN_EXEC_H
#define BEN_EXEC_H
#include <sys/types.h>
#include <unistd.h>

typedef struct exec_unit_t {
	pid_t pid;
	int infd, outfd, errfd;
	int flags;
} exec_unit;

#define EXEC_KEEP_STDIN		1
#define EXEC_KEEP_STDOUT	2
#define EXEC_KEEP_STDERR	4
#define EXEC_GROUP_LEADER	8
#define EXEC_ORPHANED		16
int exec_fork(exec_unit *u, char **argv, char **add_envp, int flags);
int exec_terminate(exec_unit *u, int *status);

int exec_daemonize(const char *inf, const char *outf, const char *errf);

int exec_signals_setup(void);

int exec_recursive_mkdir(const char *path);
int exec_path_mkdir(const char *path);
int exec_mkchdir(const char *path);

typedef struct exec_pager_t {
	pid_t pid;
	int old_out;
} exec_pager;

int exec_pager_run(exec_pager *pg);
int exec_pager_wait(exec_pager *pg);

int exec_data(char **argv, char *title, char *data);

#endif

