/*
    This file is part of ben -- a task scheduler for distributed computations
    Copyright (C) 2012-2024  Laurent Poirrier

    ben is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BEN_PARAM_H
#define BEN_PARAM_H
#include <stdio.h>
#include "buffer.h"
#include "strlist.h"
#include "config.h"

#define PARAM_STRING	0
#define PARAM_INT	1
#define PARAM_DOUBLE	2
#define PARAM_STRLIST	3
#define PARAM_SWITCH	4
#define PARAM_NEWLINE	5

// Specified by user
typedef struct param_desc_t {
	int type;
	int minpos, maxpos;
	char *imm, *immname;
	char *load, *loadname;
	int minocc, maxocc;
} param_desc;

// Filled by param_init()
typedef struct param_data_t {
	buffer buff;
	int occ;
	
	union param_value_t {
		int b;
		char *s;
		int i;
		double d;
		strlist l;
	} value;
} param_data;

typedef struct param_set_t {
	char *exec;
	int n;
	param_desc *d;
	param_data *p;
} param_set;

void param_help(FILE *f, param_set *ps);

int param_init(param_set *ps, int na, char **a);
void param_clear(param_set *ps);


#endif

